/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.dzfp.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.common.Excel.ExcleFileUtil;
import com.tofly.feesapi.common.entity.dto.PageDto;
import com.tofly.feesapi.common.entity.vo.PageResult;
import com.tofly.feesapi.common.exception.FeesErrorCode;
import com.tofly.feesapi.common.exception.FeesException;
import com.tofly.feesapi.dzfp.entity.EiFpkjRequest;
import com.tofly.feesapi.dzfp.entity.dto.*;
import com.tofly.feesapi.dzfp.service.EiFpkjRequestService;
import com.tofly.feesapi.security.model.CzyUserDetails;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * 电子发票汇总表
 *
 * @author sxb
 * @date Thu Jul 18 00:00:00 CST 2024
 */
@RestController
@AllArgsConstructor
@RequestMapping("/eifpkjrequest")
@Api(tags="电子发票-电子发票汇总表接口")
public class EiFpkjRequestController {

    private final  EiFpkjRequestService eiFpkjRequestService;

    /**
    * 分页查询
    * @param pageDto 分页对象
    * @param eiFpkjRequest 电子发票汇总表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询-开票明细查询")
    public ResultRespone <PageResult<EiFpkjRequest_select>> getEiFpkjRequestPage(@ModelAttribute PageDto pageDto, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) {
      return ResultRespone.success(eiFpkjRequestService.getPage(pageDto,eiFpkjRequest));
    }

    @GetMapping("/sum")
    @ApiOperation(value = "分页查询-开票明细查询汇总")
    public ResultRespone <List<EiFpkjRequest_sum>> listSum(HttpServletResponse response, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) throws IOException {
        List<EiFpkjRequest_sum> list = eiFpkjRequestService.getListSum(eiFpkjRequest);
        return ResultRespone.success(list);
    }

    @GetMapping("/export")
    @ApiOperation(value = "导出开票明细Excel")
    public void export(HttpServletResponse response, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) throws IOException {
        List<EiFpkjRequest_select> list = eiFpkjRequestService.getList(eiFpkjRequest);
        ExcleFileUtil.exportExcel(response, list, EiFpkjRequest_Excel.class, "用户开票明细.xlsx");
    }
    /**
     * 分页查询
     * @param page 分页对象
     * @param eiFpkjRequest 电子发票汇总表
     * @return
     */
    @GetMapping("/page/fpjf" )
    @ApiOperation(value = "分页查询-交付异常明细查询")
    public ResultRespone <Page<EiFpkjRequest_select>> getEiFpkjRequestPageFpjf(Page page, @ModelAttribute EiFpkjRequest eiFpkjRequest) {
        eiFpkjRequest.setJfYxbz("0");//未交付
        eiFpkjRequest.setClCljg("2");//
        return ResultRespone.success(eiFpkjRequestService.page(page, Wrappers.query(eiFpkjRequest)));
    }


    /**
     * 分页查询
     * @param pageDto 分页对象
     * @param eiFpkjRequest 电子发票汇总表
     * @return
     */
    @GetMapping("/sf/page" )
    @ApiOperation(value = "分页查询-水费已开票明细查询")
    public ResultRespone <PageResult<EiFpkjRequest_select>> getEiFpkjSfRequestPage(@ModelAttribute PageDto pageDto, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) {
        return ResultRespone.success(eiFpkjRequestService.getSfPage(pageDto,eiFpkjRequest));
    }
    /**
     * 分页查询
     * @param pageDto 分页对象
     * @param eiFpkjRequest 电子发票汇总表
     * @return
     */
    @GetMapping("/fp/page" )
    @ApiOperation(value = "分页查询-已开票明细查询")
    public ResultRespone <PageResult<EiFpkjRequest_select>> getEiFpkjFpRequestPage(@ModelAttribute PageDto pageDto, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) {
        return ResultRespone.success(eiFpkjRequestService.getFpPage(pageDto,eiFpkjRequest));
    }

    /**
     * 分页查询
     * @param pageDto 分页对象
     * @param eiFpkjRequest 电子发票汇总表
     * @return
     */
    @GetMapping("/znj/page" )
    @ApiOperation(value = "分页查询-违约金已开票明细查询")
    public ResultRespone <PageResult<EiFpkjRequest_select>> getEiFpkjZnjRequestPage(@ModelAttribute PageDto pageDto, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) {
        return ResultRespone.success(eiFpkjRequestService.getZnjPage(pageDto,eiFpkjRequest));
    }
    /**
     * 分页查询
     * @param pageDto 分页对象
     * @param eiFpkjRequest 电子发票汇总表
     * @return
     */
    @GetMapping("/wsf/page" )
    @ApiOperation(value = "分页查询-污水费已开票明细查询")
    public ResultRespone <PageResult<EiFpkjRequest_select>> getEiFpkjWsfRequestPage(@ModelAttribute PageDto pageDto, @ModelAttribute EiFpkjRequestFilter eiFpkjRequest) {
        return ResultRespone.success(eiFpkjRequestService.getWsfPage(pageDto,eiFpkjRequest));
    }
    /**
     * 通过id查询电子发票汇总表
     * @param
     * @return ResultRespone
     */
//    @GetMapping("/{lsbh}" )
//    @ApiOperation(value = "通过ID查询")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "lsbh", value = "ID", required = true, dataType = "String")
//    })
//    public ResultRespone getById(@PathVariable("lsbh" ) String lsbh) {
//      return ResultRespone.success(eiFpkjRequestService.getById(lsbh));
//    }

//    /**
//     * 新增电子发票汇总表
//     * @param eiFpkjRequest 电子发票汇总表
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="新增电子发票汇总表" )
//    @ApiOperation(value = "新增电子发票汇总表")
//    @PostMapping
//    public ResultRespone save(@RequestBody EiFpkjRequest eiFpkjRequest) {
//      return ResultRespone.success(eiFpkjRequestService.save(eiFpkjRequest));
//    }
    @ToFlyAppLog(title="开票查询" )
    @ApiOperation(value = "开票查询")
    @PutMapping("/kpcx")
    public ResultRespone updateSeacharById(@RequestBody EiFpkjRequest eiFpkjRequest) {
        eiFpkjRequest.setJfYxbz("0");
        eiFpkjRequest.setClLock("0");
        eiFpkjRequest.setClCljg("1");
        return ResultRespone.success(eiFpkjRequestService.updateById(eiFpkjRequest));
    }
    /**
     * 修改电子发票汇总表
     * @param
     * @return ResultRespone
     */
    @ApiOperation(value = "开票申请")
    @PutMapping("/kpsq")
    public ResultRespone updateSqById(@RequestBody EiFpkjRequest_sq eiFpkjRequest_sq) {
        EiFpkjRequest eiFpkjRequest=eiFpkjRequestService.getById(eiFpkjRequest_sq.getLsbh());
        eiFpkjRequest.setFpSfkp("1");//开票申请
        eiFpkjRequest.setJfYxbz("0");
        eiFpkjRequest.setClLock("0");
        eiFpkjRequest.setClCljg("0");
        BeanUtils.copyProperties(eiFpkjRequest_sq,eiFpkjRequest);
        return ResultRespone.success(eiFpkjRequestService.updateById(eiFpkjRequest));
    }

    @ApiOperation(value = "开票信息完善")
    @PutMapping("/kpws")
    public ResultRespone updateKpwsById(@RequestBody EiFpkjRequest_Invoice eiFpkjRequest_invoice) {
        EiFpkjRequest eiFpkjRequest=eiFpkjRequestService.getById(eiFpkjRequest_invoice.getLsbh());
        eiFpkjRequest.setFpSfkp("1");//开票申请
        BeanUtils.copyProperties(eiFpkjRequest_invoice,eiFpkjRequest);
        return ResultRespone.success(eiFpkjRequestService.updateById(eiFpkjRequest));
    }

    @ApiOperation(value = "重新交付")
    @PutMapping("/cxjf")
    public ResultRespone updateCxjfById(@RequestBody EiFpkjRequest eiFpkjRequest) {

        eiFpkjRequest.setCxJfbz("1");//重新交付
        return ResultRespone.success(eiFpkjRequestService.updateById(eiFpkjRequest));
    }

    @ApiOperation(value = "开票重试")
    @PutMapping("/kpcs")
    public ResultRespone updateKpcsById(@RequestBody EiFpkjRequest_sq eiFpkjRequest_sq) {
        EiFpkjRequest eiFpkjRequest=eiFpkjRequestService.getById(eiFpkjRequest_sq.getLsbh());
        eiFpkjRequest.setFpCkbz("1");//开票重试
        eiFpkjRequest.setClLock("0");//
        BeanUtils.copyProperties(eiFpkjRequest_sq,eiFpkjRequest);
        return ResultRespone.success(eiFpkjRequestService.updateById(eiFpkjRequest));
    }
    @ApiOperation(value = "发票下载PDF")
    @PutMapping("/fpdown")
    public ResultRespone updateFpdownById(@RequestBody EiFpkjRequest_sq eiFpkjRequest_sq) {
        EiFpkjRequest eiFpkjRequest=eiFpkjRequestService.getById(eiFpkjRequest_sq.getLsbh());
        if(eiFpkjRequest.getJfYxbz().equals("0"))
        {
            throw new FeesException("提示：该条票据信息发票未交付，不支持发票下载！", FeesErrorCode.BUSINESS_ERROR);
        }
        String StrUrl=eiFpkjRequest.getJfPdf();
        return ResultRespone.success(StrUrl);
    }
    @ApiOperation(value = "发票冲红")
    @PutMapping("/fpch")
    public ResultRespone<Boolean>  updateFpchById(@RequestBody EiFpkjRequest_sq eiFpkjRequest_sq, @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        EiFpkjRequest eiFpkjRequest=eiFpkjRequestService.getById(eiFpkjRequest_sq.getLsbh());
        if(eiFpkjRequest.getZfZfbz().equals("1")||eiFpkjRequest.getZfZfbz().equals("2"))
        {
            throw new FeesException("提示：该条票据信息发票已冲红，不支持发票冲红！", FeesErrorCode.BUSINESS_ERROR);
        }
        boolean result = eiFpkjRequestService.dzfpch(eiFpkjRequest, czyUserDetails);

        return result ? ResultRespone.success(true, "发票冲红成功") : ResultRespone.failed(false, "发票冲红失败");
    }


}
