/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.sms.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.security.model.CzyUserDetails;
import com.tofly.feesapi.sms.entity.SmsMessage;
import com.tofly.feesapi.sms.entity.dto.SmsMessageDto;
import com.tofly.feesapi.sms.entity.dto.SmsMessageYhxxDto;
import com.tofly.feesapi.sms.entity.vo.SmsMessageVo;
import com.tofly.feesapi.sms.entity.vo.SmsMessageYhxxVo;
import com.tofly.feesapi.sms.service.ISmsMessageService;
import lombok.AllArgsConstructor;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import springfox.documentation.annotations.ApiIgnore;

import java.util.Arrays;

/**
 * 短信消息
 *
 * @author sunjian
 * @date Fri Nov 08 00:00:00 CST 2024
 */
@RestController
@AllArgsConstructor
@RequestMapping("/smsmessage")
@Api(tags = "短信模块-短信消息接口")
public class SmsMessageController {

    private final ISmsMessageService ISmsMessageService;

    /**
     * 分页查询
     *
     * @param page          分页对象
     * @param smsmessageDto 抢修信息审核表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone<Page<SmsMessage>> getSmsMessagePage(Page page, @ModelAttribute SmsMessageDto smsmessageDto) {
        ResultRespone<Page<SmsMessage>> resultRespone = new ResultRespone<>();
        try {
            Page<SmsMessage> smsmessagePage = ISmsMessageService.getSmsMessagePage(page, smsmessageDto);
            resultRespone.setCode(1);
            resultRespone.setResult(smsmessagePage);
        } catch (RuntimeException e) {
            resultRespone.setCode(-1);
            resultRespone.setMessage(e.getMessage());
        }
        return resultRespone;
    }

    /**
     * 分页查询-用户信息
     *
     * @param page 分页对象
     * @param dto
     * @return
     */
    @GetMapping("/pageyhxx")
    @ApiOperation(value = "分页查询-临时公告短信用户消息")
    public ResultRespone<Page<SmsMessageYhxxVo>> getSmsMessageYhxxPage(Page page, @ModelAttribute SmsMessageYhxxDto dto) {
        ResultRespone<Page<SmsMessageYhxxVo>> resultRespone = new ResultRespone<>();
        try {
            Page<SmsMessageYhxxVo> smsmessagePage = ISmsMessageService.getSmsMessageYhxxVoList(page, dto);
            resultRespone.setCode(1);
            resultRespone.setResult(smsmessagePage);
        } catch (RuntimeException e) {
            resultRespone.setCode(-1);
            resultRespone.setMessage(e.getMessage());
        }
        return resultRespone;
    }

    /**
     * 分页查询-用户信息（包含欠费）
     *
     * @param page 分页对象
     * @param dto
     * @return
     */
    @GetMapping("/pageyhqfxx")
    @ApiOperation(value = "分页查询-用户欠费信息合计查询")
    public ResultRespone<Page<SmsMessageYhxxVo>> getSmsMessageYhqfxxPage(Page page, @ModelAttribute SmsMessageYhxxDto dto) {
        ResultRespone<Page<SmsMessageYhxxVo>> resultRespone = new ResultRespone<>();
        try {
            Page<SmsMessageYhxxVo> smsmessagePage = ISmsMessageService.getSmsMessageYhqfxxVoList(page, dto);
            resultRespone.setCode(1);
            resultRespone.setResult(smsmessagePage);
        } catch (RuntimeException e) {
            resultRespone.setCode(-1);
            resultRespone.setMessage(e.getMessage());
        }
        return resultRespone;
    }

    /**
     * 通过id查询短信消息
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id") String id) {
        return ResultRespone.success(ISmsMessageService.getById(id));
    }


    /**
     * 通过id删除短信消息
     *
     * @param id id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过ID删除短信消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable String id) {
        return ResultRespone.success(ISmsMessageService.removeById(id));
    }

    /**
     * 通过id批量删除短信消息
     *
     * @param ids id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过ID批量删除短信消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(ISmsMessageService.removeByIds(Arrays.asList(ids.split(","))));
    }

    /**
     * 通过id批量重新发送短信
     *
     * @param ids id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过id批量重新发送短信")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @PostMapping("/reSendByIds")
    public ResultRespone reSendByIds(String ids) {
        return ResultRespone.success(ISmsMessageService.reSendByIds(ids));
    }

    /**
     * 自定义发送短信
     *
     * @param phone   电话号码
     * @param content 短信内容
     * @return
     */
    @ApiOperation(value = "自定义发送短信")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "短信号码", required = true, dataType = "String"),
            @ApiImplicitParam(name = "content", value = "短信内容", required = true, dataType = "String"),
    })
    @PostMapping("/customizeSendMessage")
    public ResultRespone customizeSendMessage(String phone, String content,
                                              @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        return ResultRespone.success(ISmsMessageService.customizeSendMessage(phone, content, czyUserDetails));
    }

    /**
     * 批量发送自定义短信消息
     *
     * @param dxnr 短信内容
     * @return ResultRespone
     */
    @ApiOperation(value = "批量发送自定义短信消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dxnr", value = "短信内容", required = true, dataType = "String")
    })
    @PostMapping("/batchSendMessageYhxx")
    public ResultRespone batchSendMessageYhxx(String dxnr, @ModelAttribute SmsMessageYhxxDto dto) {
        return ResultRespone.success(ISmsMessageService.batchSendMessageYhxx(dxnr, dto));
    }

    /**
     * 批量发送模板短信消息
     *
     * @param dxnr 短信内容
     * @return ResultRespone
     */
    @ApiOperation(value = "批量发送模板短信消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dxnr", value = "模板短信内容", required = true, dataType = "String")
    })
    @PostMapping("/batchSendMessageYhqfxx")
    public ResultRespone batchSendMessageYhqfxx(String dxnr, @ModelAttribute SmsMessageYhxxDto dto) {
        return ResultRespone.success(ISmsMessageService.batchSendMessageYhqfxx(dxnr, dto));
    }
}
