package com.tofly.auth.oauth;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.tofly.auth.config.LangChaoConfig;
import com.tofly.auth.utils.RsaUtilClient;
import com.tofly.common.core.constant.CommonConstants;
import com.tofly.common.core.constant.SecurityConstants;
import com.tofly.common.core.util.IpUtil;
import com.tofly.common.core.util.SpringContextHolder;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.auth.ToflyUser;
import com.tofly.entity.pojo.User;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Collection;
import java.util.Map;
import java.util.Objects;

@Component
@AllArgsConstructor
@Slf4j
public class ToflyAuthenticationProvider implements AuthenticationProvider {
    private final UserDetailsService userDetailService;
    private final LangChaoConfig langChaoConfig;
    private final CacheManager cacheManager;

    //private final AuthTokenService authTokenService;

    private final TokenStore tokenStore;

    /**
     * 进行身份认证
     *
     * @param authentication
     * @return
     * @throws AuthenticationException
     */
    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        log.info("order 身份认证 来了。。。。。。。");
        // 获取用户输入的用户名和密码
        String username = authentication.getName();
        if(StringUtil.isEmpty(username)){
            throw new MyAuthenticationException("用户名不能为空");
        }
        String password = (String)authentication.getCredentials();
        if(StringUtil.isEmpty(password)){
            throw new MyAuthenticationException("密码不能为空");
        }
        String userId=null;
        if(username.equals("token")){
            RestTemplate restTemplate = new RestTemplate();
            HttpHeaders headers = new HttpHeaders();
            headers.set("Authorization", "Bearer "+password);
            HttpEntity entity = new HttpEntity<>(headers);
            try {
                ResponseEntity response = restTemplate.exchange(
                        langChaoConfig.getLoginUserInfo(), HttpMethod.GET, entity, String.class);
                Object body = response.getBody();
                if(body!=null){
                    String s = body.toString();
                    JSONObject jsonObject = JSON.parseObject(s);
                    Object userIdObj = jsonObject.get("userId");
                    if(userIdObj!=null){
                        userId= userIdObj.toString();
                    }
                }
            }catch (Exception e){
                e.printStackTrace();
                throw new MyAuthenticationException("门户token失效");
            }
        }
        UserDetails userDetails=null;
        Boolean flag=false;
        HttpServletRequest request = ((ServletRequestAttributes) Objects
                .requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
        if(StringUtil.isNotEmpty(userId)){
            userDetails = userDetailService.loadUserByUsername(userId);
            if(null!=userDetails && StringUtil.isNotEmpty(userDetails.getUsername())){
                username=userId;
                flag=true;
            }
        }else{
            password= RsaUtilClient.jsencryptDecryptByPrivateKeyLong(password);
            // 获取封装用户信息的对象
             userDetails = userDetailService.loadUserByUsername(username);
            // 进行密码的比对
            log.info("ip:{}",IpUtil.getIpAddr(request));
            log.info("界面传过来的password:{}",password);
            log.info("数据库查询出来的password:{}",userDetails.getPassword());

            BCryptPasswordEncoder bCryptPasswordEncoder = new BCryptPasswordEncoder();
           // log.info("用{}加密后的密码为：{}", SecurityConstants.BCRYPT+password,bCryptPasswordEncoder.encode(SecurityConstants.BCRYPT+password));
            flag = bCryptPasswordEncoder.matches(SecurityConstants.BCRYPT+password, userDetails.getPassword());
            if(StringUtil.isEmpty(password) && !flag){
                flag=true;
            }
        }

        ToflyUser toflyUser=null;
        Cache cache = cacheManager.getCache("user_details");
        if (cache != null && cache.get(username) != null && cache.get(username).get()!=null) {
            toflyUser= (ToflyUser) cache.get(username).get();
        }

        User sysUser=new User();
        sysUser.setId(toflyUser.getUserid());
        Short pwdt=toflyUser.getPwderrtimes();
        if(pwdt == null){
            pwdt=Short.valueOf("0");
        }

        if(pwdt>=5 && !CommonConstants.SUPER_ADMIN_ULEVEL.equals(toflyUser.getUserLevel())){
            //未登陆成功，清楚缓存，避免下次已经解锁登录时还是锁定状态
            cache.put(username,null);
            throw new MyAuthenticationException("账户已经被锁定，请联系管理人员");
        }

        // 校验通过
        if (flag){
            sysUser.setPwderrtimes(Short.valueOf("0"));
            sysUser.setLastIp(IpUtil.getIpAddr(request));
            SpringContextHolder.publishEvent(new UserLoginEvent(sysUser));
            RedisTokenStore redisTokenStore=(RedisTokenStore)tokenStore;
            Collection<OAuth2AccessToken> client_id = redisTokenStore.findTokensByClientIdAndUserName((String)((Map) authentication.getDetails()).get("client_id"), authentication.getName());
            client_id.forEach(oAuth2AccessToken -> {
                tokenStore.removeAccessToken(oAuth2AccessToken);
            });
            //ResultRespone resultRespone=authTokenService.removeTokenByUser(params,SecurityConstants.FROM_IN);
            //如果密码错误次数不为0，则将缓存中的次数修改为0
            if(pwdt!=0){
                ToflyUser toflyUser1=new ToflyUser(Short.valueOf("0"),toflyUser.getUserid(),toflyUser.getDepartmentId(),toflyUser.getUserLevel(),
                        toflyUser.getRealName(),toflyUser.getDepartemntName(),toflyUser.getAvatar(),
                        toflyUser.getUsername(),/*SecurityConstants.BCRYPT+*/toflyUser.getPassword(),
                        true,true,
                        true,true,toflyUser.getAuthorities());
                cache.put(username,toflyUser1);
            }

            //将权限信息也封装进去
            return new UsernamePasswordAuthenticationToken(userDetails,password,userDetails.getAuthorities());
        }



        pwdt++;
        sysUser.setPwderrtimes(pwdt);
        if(pwdt>=5){
            sysUser.setLockFlag(CommonConstants.USER_LOCK_LOCKED);
        }
        //toflyUser.setPwderrtimes(pwdt);
        //cache.put(username,toflyUser);


        //cache.put(username,toflyUser1);
        SpringContextHolder.publishEvent(new UserLoginEvent(sysUser));
        cache.put(username,null);
        String message=pwdt<5?"密码错误。当前密码错误次数是："+pwdt+",密码错误5次账户将会被锁定":
                "账户已经被锁定，请联系管理人员";
        throw new MyAuthenticationException(message);
         //验证失败返回 null
        //return null;

    }

    /**
     * 这个方法 确保返回 true 即可，
     *
     * @param aClass
     * @return
     */
    @Override
    public boolean supports(Class<?> aClass) {
        return true;
    }

    class MyAuthenticationException extends AuthenticationException{

        public MyAuthenticationException(String msg, Throwable t) {
            super(msg, t);
        }

        public MyAuthenticationException(String msg) {
            super(msg);
        }
    }
}